import Taro, { Component, Config } from '@tarojs/taro';
import { ComponentClass } from 'react';
import { View, Text, Button } from '@tarojs/components';
import { getPayOrder } from '../../api/Order/pay';
import { connect } from '@tarojs/redux';
import { Device } from '../../types/Device/Device';
import PrepayConfig from '../../types/Order/Order';
import { Customer } from '../../types/Customer/Customer';
import AES from 'crypto-js/aes';
import Utf8 from 'crypto-js/enc-utf8';
import ECBmode from 'crypto-js/mode-ecb';
import PaddingPkcs7 from 'crypto-js/pad-pkcs7';
import { paySuccess } from '../../api/Order/paySuccess';

type PageStateProps = {
  device: Device;
  prepayConfig: PrepayConfig[];
  userinfo: Customer;
};

type PageDispatchProps = {};

type PageState = {
  payId: undefined | number;
};

type Iprop = PageStateProps & PageDispatchProps;
interface Pay {
  props: Iprop;
  state: PageState;
}

@connect(({ device, prepayConfig, userinfo }) => ({
  device,
  prepayConfig,
  userinfo,
}))
class Pay extends Component {
  config: Config = {
    navigationBarTitleText: '支付金额',
  };
  constructor(props) {
    super(props);
    this.state = {
      payId: undefined,
    };
  }

  selectPayConfig(id: number) {
    this.setState({
      payId: id,
    });
  }

  payHandle() {
    const {
      device,
      userinfo: { customerId },
    } = this.props;
    const { payId } = this.state;
    if (payId) {
      getPayOrder({
        equipmentNum: device.code,
        equipmentPos: device.position,
        payType: '',
        serviceId: 0,
        smaproCustomerId: customerId,
        smaproPrepayConfigId: payId,
      })
        .then(res => {
          const { payStr, outTradeNo } = res;
          const key = customerId.toString().padEnd(16, '0');
          const payData = JSON.parse(
            AES.decrypt(payStr, Utf8.parse(key), {
              mode: ECBmode,
              padding: PaddingPkcs7,
            }).toString(Utf8),
          );
          console.log(payData);
          Taro.requestPayment({
            timeStamp: payData.msg.timeStamp.toString(),
            nonceStr: payData.msg.nonceStr,
            package: payData.msg.package,
            signType: payData.msg.signType,
            paySign: payData.msg.paySign,
          }).then(res => {
            console.log(res);
            paySuccess({ orderId: outTradeNo });
            Taro.showLoading({
              title: '设备连接中',
              mask: true,
            });
          });
        })
        .catch(err => {
          Taro.showToast({
            title: '发起支付失败!',
            icon: 'none',
          });
          console.error(err);
        });
    } else {
      Taro.showToast({
        title: '请选择预付金额',
        icon: 'none',
      });
    }
  }

  render() {
    const { device, prepayConfig } = this.props;
    const { payId } = this.state;
    return (
      <View className='Pay'>
        <View className='Pay-info'>
          <View className='Pay-info-item'>
            <Text>服务类型</Text>
            <Text>
              {prepayConfig && prepayConfig.length
                ? prepayConfig[0].serviceName
                : ''}
            </Text>
          </View>
          <View className='Pay-info-item'>
            <Text>设备编号</Text>
            <Text>{device.code}</Text>
          </View>
          <View className='Pay-info-item'>
            <Text>设备位置</Text>
            <Text>{device.position}</Text>
          </View>
          <View className='Pay-info-item'>
            <Text>适用费率</Text>
            <View className='Pay-info-rate'>
              {device.rates.map(item => (
                <View key={item.name}>
                  <Text>{item.name}</Text>
                  <Text>{item.mark}</Text>
                </View>
              ))}
              {/* <View>
                <Text>冷水</Text>
                <Text>0.02元/100ml</Text>
              </View>
              <View>
                <Text>热水</Text>
                <Text>0.02元/100ml</Text>
              </View> */}
            </View>
          </View>
        </View>
        <View className='Pay-tip'>
          温馨提示：如本次预付款未全部使用，系统将在使用结束后半小时内自动退还剩余金额，请留意查收！
        </View>
        <View className='Pay-money'>
          {prepayConfig.map(payConfig => (
            <View
              key={payConfig.id}
              className={`Pay-money-item ${
                payConfig.id == payId ? 'seleted' : ''
              }`}
              onClick={() => this.selectPayConfig(payConfig.id)}>
              {payConfig.prepayMoney}元
            </View>
          ))}
          {/* <View className='Pay-money-item'>1.00元</View>
          <View className='Pay-money-item'>2.00元</View>
          <View className='Pay-money-item'>5.00元</View>
          <Input className='Pay-money-item' placeholder='输入>0.5数额' /> */}
        </View>
        <Button className='Pay-btn' onClick={this.payHandle}>
          确定支付
        </Button>
      </View>
    );
  }
}

export default Pay as ComponentClass<Iprop, PageState>;
